'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepMapParents = exports.deepMapLeaves = exports.deepMap = undefined;

var _immutable = require('immutable');

var _utils = require('./utils');

/**
 * @module map
 */

var mapRecursive = function mapRecursive(self, childPath, mapper, options) {
  var nodePath = arguments.length <= 4 || arguments[4] === undefined ? (0, _immutable.List)() : arguments[4];

  var mapSelf = function mapSelf(self, children) {
    return mapper(self, nodePath, children);
  };

  if ((0, _utils.isLeaf)(self, childPath)) {
    return options.onlyParents ? self : mapSelf(self, (0, _immutable.List)());
  }

  var mappedChildren = self.getIn(childPath).map(function (kid, key) {
    return mapRecursive(kid, childPath, mapper, options, nodePath.push(key));
  });

  var selfWithMappedChildren = childPath.isEmpty() ? self.merge(mappedChildren) : self.setIn(childPath, mappedChildren);

  return options.onlyLeaves ? selfWithMappedChildren : mapSelf(selfWithMappedChildren, mappedChildren);
};

/**
 * Maps through all nodes in the provided tree, passing them all through a `mapper` function.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, inward from leaves to the root node.
 * Sibling nodes are processed in the order that Immutable iterates through collections.
 *
 * `TODO: note about care with child collections, note about using as reduce`
 * 
 * @param {Mapper} mapper The function to be called for every node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMap(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, {});
  };
}

/**
 * Maps through the leaf nodes in the provided tree, passing them all through a mapper function. Leaf nodes are nodes that have no child nodes.
 * Nodes are processed branch by branch, in the order that Immutable maps through the child iterables.
 *
 * @param {Mapper} mapper The function to be called for every node in the tree, the results of which will be used to create the modified tree.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapLeaves(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, { onlyLeaves: true });
  };
}

/**
 * Maps through the parent nodes in the provided tree, passing them all through a mapper function. Parent nodes are nodes that have child nodes.
 * Nodes are processed branch by branch in the order that Immutable maps through the child iterables, inward from leaves to the root node.
 *
 * @param {Mapper} mapper The function to be called for every node in the tree.
 * the results of which will be used to create the modified iterable.
 * @param {ChildPath} [childPath=null] An `Array` or `List` of keys indicating where to find each node's children from within each node.
 * @return {InputFunction} A partially applied function which accepts a single tree `Iterable`, and returns the modified tree `Iterable`.
 */

function deepMapParents(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  return function (tree) {
    return mapRecursive(tree, (0, _utils.asList)(childPath), mapper, { onlyParents: true });
  };
}

exports.deepMap = deepMap;
exports.deepMapLeaves = deepMapLeaves;
exports.deepMapParents = deepMapParents;