import tape from 'tape';
import {
  is,
  fromJS,
  Iterable,
  List,
  Map,
  OrderedMap
} from 'immutable';

import {
  deepMap,
  deepMapLeaves,
  deepMapParents
} from '../map';

import {
  isLeaf
} from '../utils';

const nestedMaps = fromJS({
  a: {
    b: "B",
    c: "C",
    d: {
      e: "E"
    }
  },
  f: "F"
});

const childLists = fromJS({
  name: "root",
  children: [
    {
      name: "child 1",
      children: [
        {name: "grandchild 1"},
        {name: "grandchild 2"}
      ]
    },
    {
      name: "child 2",
      children: []
    },
    {name: "child 3"}
  ]
});

tape('deepMap', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepMap(item => item.set('abc', 123), ['children'])),
    	fromJS({
		  name: "root",
		  abc: 123,
		  children: [
		    {
		      name: "child 1",
		      abc: 123,
		      children: [
		        {name: "grandchild 1", abc: 123},
		        {name: "grandchild 2", abc: 123}
		      ]
		    },
		    {
		      name: "child 2",
		      abc: 123,
		      children: []
		    },
		    {name: "child 3", abc: 123}
		  ]
		})
    ), 'deepMap iterates through every node using a childPath');

    test.ok(is(
    	nestedMaps.update(deepMap(item => {
    		// normally deepMap on trees that don't require childPaths is a bit pointless
    		// because you'll almost certainly have to deal with leaves and parents differently
    		return isLeaf(item) ? item + "!" : item.set('abc', 123);
    	})),
    	fromJS({
		  a: {
		    b: "B!",
		    c: "C!",
		    d: {
		      e: "E!",
		      abc: 123
		    },
		    abc: 123
		  },
		  f: "F!",
		  abc: 123
		})
    ), 'deepMap iterates through every node without using a childPath (kind of pointless)');
});


tape('deepMapLeaves', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepMapLeaves(item => item.set('abc', 123), ['children'])),
    	fromJS({
		  name: "root",
		  children: [
		    {
		      name: "child 1",
		      children: [
		        {name: "grandchild 1", abc: 123},
		        {name: "grandchild 2", abc: 123}
		      ]
		    },
		    {
		      name: "child 2",
		      abc: 123,
		      children: []
		    },
		    {name: "child 3", abc: 123}
		  ]
		})
    ), 'deepMapLeaves iterates through only leaves when using a childPath');

    test.ok(is(
    	nestedMaps.update(deepMapLeaves(item => item + "!")),
    	fromJS({
		  a: {
		    b: "B!",
		    c: "C!",
		    d: {
		      e: "E!"
		    }
		  },
		  f: "F!"
		})
    ), 'deepMapLeaves iterates through only leaves when not using a childPath');
});

tape('deepMapParents', test => {
    test.plan(2);

    test.ok(is(
    	childLists.update(deepMapParents(item => item.set('abc', 123), ['children'])),
    	fromJS({
		  name: "root",
		  abc: 123,
		  children: [
		    {
		      name: "child 1",
		      abc: 123,
		      children: [
		        {name: "grandchild 1"},
		        {name: "grandchild 2"}
		      ]
		    },
		    {
		      name: "child 2",
		      children: []
		    },
		    {name: "child 3"}
		  ]
		})
    ), 'deepMapParents iterates through every parent node using a childPath');

    test.ok(is(
    	nestedMaps.update(deepMapParents(item => item.set('abc', 123))),
    	fromJS({
		  a: {
		    b: "B",
		    c: "C",
		    d: {
		      e: "E",
		      abc: 123
		    },
		    abc: 123
		  },
		  f: "F",
		  abc: 123
		})
    ), 'deepMapParents iterates through every parent node without using a childPath');
});
